"""
加密用公钥
揭秘用私钥
"""

import os
import base64
import logging
import traceback
from Crypto import Random
from Crypto.Cipher import PKCS1_v1_5
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_OAEP


BASE_DIR = os.path.abspath(os.path.dirname(__file__))
PRIVATE_KEY = os.path.join(BASE_DIR, "private_key.pem")
PUBLIC_KEY = os.path.join(BASE_DIR, "public_key.pem")


def generator_key():
    """
    生成密钥对
    :return:
    """
    # random_generator = Random.new().read
    # key = RSA.RSA.generate(1024, random_generator)
    key = RSA.generate(1024)
    file_out = open(PRIVATE_KEY, "wb")
    private_key = key.export_key()
    file_out.write(private_key)

    public_key = key.publickey().export_key()
    file_out = open(PUBLIC_KEY, "wb")
    file_out.write(public_key)


def get_key(key, public=True, need_bytes=False):
    """
    获取公钥
    :param need_bytes:
    :param key:
    :param public:
    :return:
    """
    try:
        if not (os.path.exists(PUBLIC_KEY) and
                os.path.exists(PRIVATE_KEY)):
            logging.debug("youyouyou")
            generator_key()
        if not os.path.exists(key):
            return open(
                PUBLIC_KEY if public else PRIVATE_KEY,
                "rb" if need_bytes else "r").read()
        else:
            return open(key, "rb" if need_bytes else "r").read()
    except Exception as err:
        logging.error(str(err))
        logging.error(traceback.format_exc())
        return None


def encrypt_data(data, public_key):
    """
    用公钥进行加密
    :param data:
    :param public_key:
    :return:
    """
    if isinstance(data, str):
        data = bytes(data, encoding="utf8")
    elif not isinstance(data, bytes):
        return None
    key = get_key(public_key, need_bytes=True)
    if not key:
        return None
    try:
        key = RSA.import_key(key)
        cipher = PKCS1_v1_5.new(key)
        return cipher.encrypt(data)
    except Exception as err:
        logging.error(str(err))
        logging.error(traceback.format_exc())
        return None


def decrypt_data(data, private_key):
    """
    用私钥进行解密
    :param data:
    :param private_key:
    :return:
    """
    if isinstance(data, str):
        data = base64.b64decode(bytes(data, encoding="utf8"))
    elif not isinstance(data, bytes):
        return None
    key = get_key(private_key, need_bytes=True)
    if not key:
        return None
    try:
        key = RSA.import_key(key)
        cipher = PKCS1_v1_5.new(key)
        length = len(data)
        default_length = 128
        if length < 128:
            return cipher.decrypt(data, sentinel='error').decode("utf8")
        else:
            offset = 0
            res = []
            while length - offset > 0:
                if length - offset > default_length:
                    res.append(cipher.decrypt(data[offset: offset + default_length], sentinel='error'))
                else:
                    res.append(cipher.decrypt(data[offset:], sentinel='error'))
                offset += default_length
            return b''.join(res).decode("utf8")
    except Exception as err:
        logging.error(str(err))
        logging.error(traceback.format_exc())
        return None

