#! /usr/bin/python3
# coding:utf8

"""
此处应该定义如下API

    /app/user 负责用户的注册和登录
                post请求：
                    负载中 action 决定注册还是登录
    /app/article[/article_id:int] get不带id负责返回不带内容的文章基础信息列表
                                  get带id返回带详细内容的一篇文章
                                  post不带id则代表保存文章
                                  post带id则代表
                                  delete 代表删除文章
    /app/public_key 负责获取公钥加密密码
    /app/tag /article_id:int  get不带id负责返回不带内容的标签基础信息列表
                              get带id返回带详细内容的一个标签
                              post不带id则代表保存标签
                              post带id则代表更新次标签
                              delete 代表删除标签

"""

import os
from django.shortcuts import render
import json
import traceback
import logging
import copy
import time
# Create your views here.
from django.http import HttpResponse, FileResponse
from rest_framework import generics, mixins, viewsets
from rest_framework.views import APIView
from rest_framework.response import Response
from django.core.paginator import Paginator
from . import models
from . import serializers
from .utils.rsa_crypt import get_key, PUBLIC_KEY, decrypt_data, PRIVATE_KEY
import hashlib


BASE_DIR = os.path.abspath(os.path.dirname(__file__))


def register(request):
    return HttpResponse("<center><p>欢迎欢迎，您现在处于注册用户界面。</p></center>")


def json_dumps(data):
    try:
        return json.dumps(data, indent=4, ensure_ascii=False)
    except Exception as err:
        logging.error(data)
        logging.error(str(err))
        logging.error(traceback.format_exc())
        return '{"message": "错误的Json格式，请系统管理员检查错误！"}'


# class ArticleList(generics.ListCreateAPIView):
#     queryset = models.Article.objects.all().order_by('id')
#     serializer_class = serializers.ArticleSerializer

# class ArticleList(mixins.ListModelMixin,
#                   mixins.CreateModelMixin,
#                   generics.GenericAPIView):
#     queryset = models.Article.objects.all()
#     serializer_class = serializers.ArticleSerializer
#
#     def get(self, request, *args, **kwargs):
#         return self.list(request, *args, **kwargs)
#
#     def post(self, request, *args, **kwargs):
#         return self.create(request, *args, **kwargs)


# class ArticleDetail(generics.RetrieveUpdateDestroyAPIView):
#     queryset = models.Article.objects.all().order_by("id")
#     serializer_class = serializers.ArticleSerializer

# class ArticleDetail(mixins.RetrieveModelMixin,
#                     mixins.UpdateModelMixin,
#                     mixins.DestroyModelMixin,
#                     generics.GenericAPIView):
#     queryset = models.Article.objects.all()
#     serializer_class = serializers.ArticleSerializer
#
#     def get(self, request, *args, **kwargs):
#         return self.retrieve(request, *args, **kwargs)
#
#     def put(self, request, *args, **kwargs):
#         return self.update(request, *args, **kwargs)
#
#     def delete(self, request, *args, **kwargs):
#         return self.destroy(request, *args, **kwargs)


class ArticleViewSet(viewsets.ModelViewSet):
    queryset = models.Article.objects.all().order_by("-createTime")
    serializer_class = serializers.ArticleSerializer

    def list(self, request, *args, **kwargs):
        try:
            label_id = request.query_params.get('value', 0)
            if label_id:
                label_id = int(label_id[0])
            else:
                label_id = 0
        except Exception as err:
            label_id = 0
        if not label_id:
            queryset = models.Article.objects.all().order_by("-createTime")
        else:
            queryset = models.Article.objects.all().filter(label=label_id).order_by("-createTime")

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = serializers.ArticleListSerializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = serializers.ArticleListSerializer(queryset, many=True)
        return Response(serializer.data)


class LabelViewSet(viewsets.ModelViewSet):
    queryset = models.Label.objects.all().order_by("id")
    serializer_class = serializers.LabelSerializer

    def list(self, request, *args, **kwargs):
        queryset = self.queryset

        serializer = serializers.LabelSerializer(queryset, many=True)
        return Response(serializer.data)


class KeyAPI(APIView):
    """
    获取公钥
    """
    return_data = {
        "message": None,
        "status": 0,
        "data": None
    }

    def get(self, request):
        """
        直接返回公钥
        :param request:
        :return:
        """
        key = get_key(PUBLIC_KEY)
        if not key:
            self.return_data["message"] = "公钥获取失败！"
            self.return_data["status"] = 1
        else:
            self.return_data["data"] = key
        return HttpResponse(json_dumps(self.return_data))


def make_token(user):
    """
    制作用户的token
    :param user:
    :return:
    """
    now = str(time.time())
    verify = hashlib.sha1(bytes(now+user.username,
                                encoding="utf-8")).hexdigest()[:16]
    token = models.Token()
    token.token = verify
    token.user = user
    token.save()
    return verify


class UserAPI(APIView):
    """
    管理用户登陆注册的API
    """
    return_template = {
        "message": None,
        "status": 0,
        "data": None
    }

    def post(self, request):
        """
        注册或者登陆
        :param request:
        :return:
        """
        self.return_data = copy.deepcopy(self.return_template)
        action = request.data.get("action")
        logging.debug(request.data)
        if action == "register":
            return self.register(request)
        elif action == "login":
            return self.login(request)
        elif action == "token":
            return self.verify_token(request)
        else:
            self.return_data["message"] = "错误请求！"
            self.return_data["status"] = 1
            return HttpResponse(json_dumps(self.return_data))

    def register(self, request):
        """
        用户注册
        :param request:
        :return:
        """
        data = copy.deepcopy(request.data)
        keys = ("username", "password", "email")
        verify = dict()
        for key in keys:
            verify[key] = data.get(key)
        if not all(verify):
            self.return_data["message"] = "参数错误！"
            self.return_data["status"] = 1
            return HttpResponse(json_dumps(self.return_data))
        verify["password"] = decrypt_data(verify["password"], PRIVATE_KEY)
        data["repeat_password"] = decrypt_data(data["repeat_password"], PRIVATE_KEY)
        if verify["password"] != data["repeat_password"]:
            self.return_data["message"] = "两次密码不一致！"
            self.return_data["status"] = 1
            return HttpResponse(json_dumps(self.return_data))
        user = models.User()
        user.username = verify["username"]
        user.password = verify["password"]
        user.email = verify["email"]
        try:
            user.save()
            self.return_data["message"] = "注册成功！"
        except Exception as err:
            logging.error(str(err))
            logging.error(traceback.format_exc())
            self.return_data["message"] = "验证失败！"
            self.return_data["status"] = 1
        return HttpResponse(json_dumps(self.return_data))

    def login(self, request):
        """
        用户注册
        :param request:
        :return:
        """
        data = copy.deepcopy(request.data)
        data["password"] = decrypt_data(data["password"], PRIVATE_KEY)
        data["username"] = data.get("username")
        if data["password"] and data["username"]:
            try:
                user = models.User.objects.filter(
                    username=data["username"],
                    password=data["password"]).first()
                if user:
                    if data.get("rememberMe") is True:
                        self.return_data["message"] = "登陆成功！"
                        self.return_data["token"] = make_token(user)
                        self.return_data["user"] = serializers.UserSerializer(user).data
                    else:
                        self.return_data["message"] = "登陆成功！"
                        self.return_data["token"] = make_token(user)
                        self.return_data["user"] = serializers.UserSerializer(user).data
                    return HttpResponse(json_dumps(self.return_data))
            except Exception as err:
                logging.error(str(err))
                logging.error(traceback.format_exc())
        self.return_data["status"] = 1
        self.return_data["message"] = "用户名或密码错误！"
        return HttpResponse(json_dumps(self.return_data))

    def verify_token(self, request):
        """
        直接验证token是否存在或者存在但是过期
        :param request:
        :return:
        """
        verify = request.data.get("token")
        if not verify:
            self.return_data["message"] = "登录信息不存在或者已经过期！"
            self.return_data["status"] = 1
            return HttpResponse(json_dumps(self.return_data))
        token = models.Token.objects.filter(token=verify).first()
        if ((not token) or
                (time.time() - token.createTime.timestamp() > 7*24*60*60)):
            self.return_data["message"] = "登录信息不存在或者已经过期！"
            self.return_data["status"] = 1
            return HttpResponse(json_dumps(self.return_data))
        user = token.user
        self.return_data["user"] = serializers.UserSerializer(user).data
        self.return_data["token"] = token.token
        now = time.time()
        return HttpResponse(json_dumps(self.return_data))


class ImageAPI(APIView):
    """
    获取公钥
    """
    return_data = {
        "message": None,
        "status": 0,
        "data": None
    }

    def get(self, request, image=None):
        """
        直接返回图片
        :param request:
        :return:
        """
        data = copy.deepcopy(self.return_data)
        file_path = os.path.join(BASE_DIR, "images", image or '')
        if not os.path.isfile(file_path):
            data["status"] = 1
            data["message"] = "图片已经不存在了！"
            return HttpResponse(json_dumps(data))
        else:
            res =  FileResponse(open(file_path, "rb"))
            res["Content-Type"] = "image/png"
            res["Content-Disposition"] = "inline"
            return res

    def post(self, request):
        """
        制作图片外链
        """
        data = copy.deepcopy(self.return_data)
        image = request.FILES.get("file")
        if not image:
            data["status"] = 1
            data["message"] = "图片上传失败！"
            return HttpResponse(json_dumps(data))
        filename = str(time.time())
        file_path = os.path.join(BASE_DIR, "images", filename)
        with open(file_path, "wb") as fd:
            fd.write(image.read())

        data["filename"] = filename
        return HttpResponse(json_dumps(data))
