import Vue              from 'vue'
import VueResource      from 'vue-resource'
import App              from './App'
import router           from './router'
import store            from './store'
import mavonEditor      from 'mavon-editor'
import ImgInputer       from 'vue-img-inputer'
import 'vue-img-inputer/dist/index.css'
import 'mavon-editor/dist/css/index.css'
import './assets/css/index.scss'

function getCookie (name) {
    let cookieValue = null
    if (document.cookie && document.cookie !== '') {
        const cookies = document.cookie.split(';')
        for (let i = 0; i < cookies.length; i++) {
            const cookie = cookies[i].trim()
            // Does this cookie string begin with the name we want?
            if (cookie.substring(0, name.length + 1) === (name + '=')) {
                cookieValue = decodeURIComponent(cookie.substring(name.length + 1))
                break
            }
        }
    }
    return cookieValue
}

const csrftoken = getCookie('csrftoken')

Vue.config.productionTip = false

Vue.use(VueResource)
Vue.use(mavonEditor)
Vue.component('ImgInputer', ImgInputer)

Vue.filter('toDate', (date) => {                    // 2017年5月10日15：35
    if (date) {
        const d = new Date(date)
        const minutes = d.getMinutes() < 10 ? '0' + d.getMinutes() : d.getMinutes()
        const hours = d.getHours() < 10 ? '0' + d.getHours() : d.getHours()
        return d.getFullYear() + '年' + (d.getMonth() + 1) + '月' +
            d.getDate() + '日 ' + hours + ' : ' + minutes
    }
})

Vue.filter('to_date', (date) => {                   // 2017-5-10 at 15：35
    if (date) {
        const d = new Date(date)
        const minutes = d.getMinutes() < 10 ? '0' + d.getMinutes() : d.getMinutes()
        const hours = d.getHours() < 10 ? '0' + d.getHours() : d.getHours()
        return d.getFullYear() + '-' + (d.getMonth() + 1) + '-' +
            d.getDate() + ' ' + hours + ': ' + minutes
    }
})

// Vue.filter('toTag', (arr) => {
//     if (arr) {
//         return arr.join('，')
//     }
// })

Vue.filter('toTag', (arr) => {
    if (arr) {
        for (let tag of store.state.tags) {
            if (tag.id === arr) {
                return tag.tags
            }
        }
        return arr
    }
})

Vue.http.interceptors.push((request, next) => {
    if (window.localStorage.getItem('token')) {
        request.headers.set('authorization', 'Bearer ' + window.localStorage.getItem('token'))
    }
    if (csrftoken) {
        request.headers.set('X-CSRFToken', csrftoken)
    }
    next((response) => {
        if (response.status === 401) {
            store.commit('unset_user')
            router.go({name: 'login'})
        }
        return response
    })
})

/* eslint-disable no-new */
let app = new Vue({
    el: '#app',
    router,
    store,
    template: '<App/>',
    components: { App }
})

console.log(app)
