import Vue      from 'vue'
import router   from '../router'
import JSEncrypt from 'jsencrypt'
// import Store from './index'

// JSEncrypt加密方法(参数1:数据，参数2：公钥)
// eslint-disable-next-line camelcase
function encrypt (data, pub_key) {
    let e = new JSEncrypt()
    e.setPublicKey(pub_key)
    return e.encrypt(data.toString())
}

// let server = 'http://192.168.1.3:8000'
let server = ''

// 当发起请求数据后 设置一个状态确认正在后台请求  以确保频繁请求
const beginLoading = (commit, add) => {
    add ? commit('loadMore_toggle', true) : commit('isLoading_toggle', true)
    return Date.now()
}

const endLoading = (commit, startTime, toggle) => {
    const leftTime = 500 - (Date.now() - startTime)
    leftTime > 0 ? setTimeout(commit(toggle, false), leftTime) : commit(toggle, false)
}

export default {
    loginBefore ({commit, state}) {
        if (!state.token) {
            let token = localStorage.getItem('token')
            if (!token) {
                return {status: 1, message: '请登录...'}
            } else {
                return Vue.http.post(server + '/app/user',
                    {action: 'token',
                        token: token})
                    .then((resp) => {
                        return resp.data
                    })
                    .catch((err) => { console.log(err) })
            }
        }
    },
    login ({commit, state}, payload) {
        let pw = payload.password
        return Vue.http.get(server + '/app/public_key')
            .then((response) => {
                state.publicKey = response.data.data
                pw = encrypt(pw, state.publicKey)
                return Vue.http.post(server + '/app/user',
                    {action: 'login',
                        username: payload.username,
                        password: pw})
                    .then((resp) => {
                        return resp.data
                    })
                    .catch((err) => { console.log(err) })
            }).catch((err) => { console.log(err) })
        // return Vue.http.post(server + '/app/user', {username: name, password: pw}).catch((err) => { console.log(err) })
    },
    register ({commit}, payload) {
        return Vue.http.post(server + '/app/user', payload).catch((err) => { console.log(err) })
    },
    resetUser ({commit}, payload) {
        // return Vue.http.post('/app/user', payload)
        //     .then(() => {
        //         commit('unset_user')
        //         router.go({name: 'login'})
        //     }).catch((err) => { console.log(err) })
        return () => {
            commit('unset_user')
            router.go({name: 'login'})
        }
    },
    //  article的http请求
    saveArticle ({state, commit}, aid) {
        let article = {}
        article.title = state.article.title
        article.label = state.article.tags
        article.content = state.article.content
        article.isDeleted = false
        article.author = state.user.id
        article.isPrimary = false
        commit('isSaving_toggle', false)
        if (aid) {
            // 保存修改内容
            return Vue.http.put(server + '/app/article/' + aid + '/', article)
                .then(() => {
                    commit('isSaving_toggle', true)
                    router.push({name: 'posts'})
                }, () => { alert('保存失败') }).catch((err) => { console.log(err) })
        } else {
            // 提交新文章
            return Vue.http.post(server + '/app/article/', article)
                .then(() => {
                    commit('isSaving_toggle', true)
                    router.push({name: 'posts'})
                }, () => { alert('保存失败') }).catch((err) => { console.log(err) })
        }
    },
    getAllArticles ({commit, state}, payload) {
        commit('moreArticle_toggle', true)
        const startTime = beginLoading(commit, payload.add)
        if (payload.value !== state.curTag.id) {
            commit('isLoading_toggle', false)
        }
        return Vue.http.get(server + '/app/article', {params: payload})
            .then(response => response.json())
            .then(articles => {
                state.previousPage = articles.previous
                state.nextPage = articles.next
                if (!articles.next) {
                    commit('moreArticle_toggle', false)
                    commit('noMore_toggle', true)
                } else {
                    commit('noMore_toggle', false)
                }
                for (let article of articles.results) {
                    article['date'] = article.createTime
                    article['tags'] = article.label
                    article['isPublish'] = article.isDeleted
                    article['aid'] = article.id
                }
                if (payload.add) {
                    commit('add_articles', articles.results)
                    endLoading(commit, startTime, 'loadMore_toggle')
                } else {
                    commit('set_all_articles', articles.results)
                    endLoading(commit, startTime, 'isLoading_toggle')
                }
            }).catch((err) => { console.log(err) })
    },
    getArticle ({commit, state}, aid) {
        const startTime = beginLoading(commit, false)
        if (router.currentRoute.hash) {
            commit('isLoading_toggle', false)
        }
        document.title = '加载中...'
        // 获取单篇文章所有内容
        return Vue.http.get(server + '/app/article/' + aid)
            .then(response => {
                let article = response.data
                article['date'] = article.createTime
                article['tags'] = article.label
                article['isPublish'] = article.isDeleted
                article['aid'] = article.id
                commit('set_article', article)
                commit('set_headline', {content: state.article.title, animation: 'animated rotateIn'})
                document.title = state.article.title
                endLoading(commit, startTime, 'isLoading_toggle')
            }).catch((err) => { console.log(err) })
    },
    delArticle ({dispatch}, payload) {
        // 使用delete删除文章
        return Vue.http.delete(server + '/app/article/' + payload.aid)
            .then(() => {
                if (payload.route.name === 'posts') dispatch('getAllArticles', {page: payload.page, limit: 8})
                if (payload.route.name === 'drafts') dispatch('getAllDrafts', {page: payload.page, limit: 8})
                if (payload.route.name === 'search') router.push({name: 'posts'})
            }).catch((err) => { console.log(err) })
    },
    // draft的http请求
    saveDraft ({state, commit}, aid) {
    // 可能要改saveArticle
        if (aid) {
            return Vue.http.patch(server + '/app/draft/' + aid, state.article)
                .then(() => {
                    commit('isSaving_toggle', true)
                    router.push({name: 'drafts'})
                }, () => { alert('保存失败') }).catch((err) => { console.log(err) })
        } else {
            return Vue.http.post(server + '/app/draft', state.article)
                .then(() => {
                    commit('isSaving_toggle', true)
                    router.push({name: 'drafts'})
                }, () => { alert('保存失败') }).catch((err) => { console.log(err) })
        }
    },
    getAllDrafts ({commit}, payload) {
        return Vue.http.get(server + '/app/drafts', {params: {payload}})
            .then(response => response.json())
            .then(articles => {
                commit('set_all_articles', articles)
            }).catch((err) => { console.log(err) })
    },
    // search
    searchArticles ({commit}, payload) {
        document.title = '搜索中...'
        commit('moreArticle_toggle', true)
        const startTime = beginLoading(commit, payload.add)
        // return Vue.http.get(server + '/app/someArticles', {params: {payload}})
        return Vue.http.get(server + '/app/someArticles', payload)
            .then(response => response.json())
            .then(articles => {
                if (articles.length === 0) {
                    commit('moreArticle_toggle', false)
                    commit('noMore_toggle', true)
                } else {
                    commit('noMore_toggle', false)
                }
                if (payload.add) {
                    commit('add_articles', articles)
                    endLoading(commit, startTime, 'loadMore_toggle')
                } else {
                    commit('set_all_articles', articles)
                    endLoading(commit, startTime, 'isLoading_toggle')
                }
                document.title = '搜索成功'
            }).catch((err) => { console.log(err) })
    },
    // tags
    getAllTags ({commit}) {
        return Vue.http.get(server + '/app/tags')
            .then(response => {
                let tags = response.data
                for (let label of tags) {
                    label['tags'] = label.label
                }
                commit('set_tags', tags)
            }).catch((err) => { console.log(err) })
    },
    // email
    sendMail ({commit}, payload) {
        return Vue.http.post(server + '/app/mail', payload).catch((err) => { console.log(err) })
    },
    // comment
    summitComment ({commit}, payload) {
        return Vue.http.post(server + '/app/comment', payload)
    },
    getAllComments ({commit}, payload) {
        return Vue.http.get(server + '/app/comment', {params: {payload}})
                .then(response => response.json())
                .then(comments => {
                    commit('set_comments', comments)
                }).catch((err) => { console.log(err) })
    },
    updateLike ({commit}, payload) {
        return Vue.http.patch(server + '/app/comment/' + payload.id, {option: payload.option})
                .catch((err) => { console.log(err) })
    }
}
